// Copyright 1998-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
// $Header: /Network/Source/CVS/OmniGroup/OmniFoundation/Formatters.subproj/OFSocialSecurityFormatter.m,v 1.1 1999/04/14 09:38:20 ryan Exp $

#import "OFSocialSecurityFormatter.h"

#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>

@implementation OFSocialSecurityFormatter

- (NSString *)stringForObjectValue:(id)object;
{
    return object;
}

- (BOOL)getObjectValue:(id *)anObject forString:(NSString *)string errorDescription:(NSString **)error;
{
    if (!anObject)
        return YES;

    if (![string length]) {
        *anObject = nil;
        return YES;
    } else if ([string length] < 11) {
        if (error)
            *error = @"That is not a valid social security number.";
        *anObject = nil;
        return NO;
    } else {
        *anObject = string;
    }
    return YES;
}

enum SocialSecurityState {
    ScanFirstPart, ScanSecondPart, ScanThirdPart, ScanDash, Done
};

- (BOOL)isPartialStringValid:(NSString *)partialString newEditingString:(NSString **)newString errorDescription:(NSString **)error;
{
    unsigned int length = [partialString length];
    unsigned int index;
    unsigned int digits = 0;
    enum SocialSecurityState state = ScanFirstPart;
    enum SocialSecurityState previousState = -1;
    unichar result[20];
    unichar *resultPtr = result;
    unichar c;
    BOOL changed = NO;

    for (index = 0; index < length; index++) {
        c = [partialString characterAtIndex:index];

        switch(state) {
            case ScanFirstPart:
                if ((c >= '0') && (c <= '9')) {
                    *resultPtr++ = c;
                    if (++digits == 3) {
                        state = ScanDash;
                        previousState = ScanFirstPart;
                    }
                } else {
                    changed = YES;
                }
                break;
            case ScanSecondPart:
                if ((c >= '0') && (c <= '9')) {
                    *resultPtr++ = c;
                    if (++digits == 2) {
                        state = ScanDash;
                        previousState = ScanSecondPart;
                    }
                } else {
                    changed = YES;
                }
                break;
            case ScanDash:
                if (c == '-') {
                    *resultPtr++ = c;
                    if (previousState == ScanFirstPart) {
                        state = ScanSecondPart;
                    } else {
                        state = ScanThirdPart;
                    }
                    previousState = ScanDash;
                    digits = 0;
                } else if ((c >= '0') && (c <= '9')) {
                    *resultPtr++ = '-';
                    *resultPtr++ = c;
                    if (previousState == ScanFirstPart) {
                        state = ScanSecondPart;
                    } else {
                        state = ScanThirdPart;
                    }
                    previousState = ScanDash;
                    digits = 1;
                    changed = YES;
                } else {
                    changed = YES;
                }
                break;
            case ScanThirdPart:
                if ((c >= '0') && (c <= '9')) {
                    *resultPtr++ = c;
                    if (++digits == 4) {
                        state = Done;
                        previousState = ScanThirdPart;
                    }
                } else {
                    changed = YES;
                }
                break;
            case Done:
                changed = YES;
                break;
        }
    }
    if (changed)
        *newString = [NSString stringWithCharacters:result length:(resultPtr - result)];
    return !changed;
}

@end
