// Copyright 1998-1999 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "NSNotificationQueue-OFExtensions.h"

#import <Foundation/Foundation.h>
#import <OmniBase/OmniBase.h>

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/OmniFoundation/OpenStepExtensions.subproj/NSNotificationQueue-OFExtensions.m,v 1.3 1999/03/30 21:25:45 kc Exp $")

@implementation NSNotificationQueue (OFExtensions)

- (void) enqueueNotificationName: (NSString *) name
                          object: (id) object
                    postingStyle: (NSPostingStyle) postingStyle;
{
    NSNotification *notification;

    notification = [NSNotification notificationWithName: name object: object];

    [self enqueueNotification: notification postingStyle: postingStyle];
}

- (void) enqueueNotificationName: (NSString *) name
                          object: (id) object
                        userInfo: (NSDictionary *) userInfo
                    postingStyle: (NSPostingStyle) aStyle;
{
    NSNotification *notification;

    notification = [NSNotification notificationWithName: name
                                                 object: object
                                               userInfo: userInfo];

    [self enqueueNotification: notification postingStyle: aStyle];
}

- (void) enqueueNotificationName: (NSString *) name
                          object: (id) object
                        userInfo: (NSDictionary *) userInfo
                    postingStyle: (NSPostingStyle) aStyle
                    coalesceMask: (unsigned) coalesceMask
                        forModes: (NSArray *) modes;
{
    NSNotification *notification;

    notification = [NSNotification notificationWithName: name
                                                 object: object
                                               userInfo: userInfo];

    [self enqueueNotification: notification
                 postingStyle: aStyle
                 coalesceMask: coalesceMask
                     forModes: modes];
}

- (void) dequeueNotificationsMatching: (NSString *) name
                               object: (id) object
                         coalesceMask: (unsigned) coalesceMask;
{
    NSNotification *notification;

    notification = [NSNotification notificationWithName: name
                                                 object: object];

    [self dequeueNotificationsMatching: notification
                          coalesceMask: coalesceMask];
}

- (void) firePendingNotifications
{
    NSRunLoop *runLoop;
    NSString  *mode;

    // This hack depends upon the fact that when you call -limitDateForMode: on NSRunLoop
    // it will do some private API goop with NSNotificationQueue to post any pending
    // notification.
    
    runLoop = [NSRunLoop currentRunLoop];
    if (!(mode = [runLoop currentMode]))
        mode = NSDefaultRunLoopMode;
    [runLoop limitDateForMode: mode];
}

@end
